# [Home](https://github.com/babyfish-ct/kimmer/)/[kimmer](./README.md)/Draft properties VS Draft functions

Let's see an immutable interface

```kt
interface Book : Immutable {
    val name: String  // scalar
    val store: BookStore? // association
    val authors: List<Author> // assocaition
}
```

The ksp pre-compiler of kimmer will generate the mutable interface for it
```kt
// This file is generated by ksp
interface BookDraft<T : Book> : Book, Draft<T> {

    override var name: String 

    override var store: BookStore? // Draft property
    fun store(): BookStoreDraft<out BookStore> // Draft function

    override var authors: List<Author> // Draft property
    fun authors(): MutableList<AuthorDraft<out Author>> // Draft function
}
```

As we can see, "BookDraft" overrides all read-only properties in Book with writable properties. 

In addition, we can also find that not only properties but also functions are generated.

Apart from the obvious that properties can be modified and functions can only take values, what is the difference between draft properties and draft functions?

|     |Draft property Getter | Draft function |
|-----|------|-----|
|Behavior| Get | Get or Create |
|Nullity| May return null | Never return null |
|UnloadedException| May throw exception | Never throw exception |
|Collection Type| List | MutableList |
|Element/Reference Type|Immutable|Draft|

### Should I use draft property or a draft function?

1. If you want to replace the entire association, please use the draft property
```kt
val book = new(Book::class).by {
    authors = listOf(
        new(Author::class).by { 
            name = "Autor1"
        },
        new(Author::class).by { 
            name = "Autor2"
        }
    )
}
```

2. If you want to modify an existing association, please use the draft function
```kt
val book = new(Book::class).by {
    authors() += new(Author::class).by { 
        name = "Autor1"
    }
    authors() += new(Author::class).by { 
        name = "Autor2"
    }
}
```

---------------------------------

[< Previous: Dynamics & unlodead properties](./dynamic.md) | [Back to document](./README.md) | [Next: Jackson >](./jackson.md)
